@@ -295,6 +295,7 @@ main (int   argc,
     gchar *filename;
     gchar *dirname;
   } dirname_checks[] = {
+#ifndef NATIVE_WIN32
     { "/", "/" },
     { "////", "/" },
     { ".////", "." },
@@ -306,8 +307,26 @@ main (int   argc,
     { "a/b", "a" },
     { "a/b/", "a/b" },
     { "c///", "c" },
+#else
+    { "\\", "\\" },
+    { ".\\\\\\\\", "." },
+    { ".", "." },
+    { "..", "." },
+    { "..\\", ".." },
+    { "..\\\\\\\\", ".." },
+    { "", "." },
+    { "a\\b", "a" },
+    { "a\\b\\", "a\\b" },
+    { "c\\\\\\", "c" },
+#endif
   };
   guint n_dirname_checks = sizeof (dirname_checks) / sizeof (dirname_checks[0]);
+  guint16 gu16t1 = 0x44afU, gu16t2 = 0xaf44U;
+  guint32 gu32t1 = 0x02a7f109U, gu32t2 = 0x09f1a702U;
+#ifdef G_HAVE_GINT64
+  guint64 gu64t1 = G_GINT64_CONSTANT(0x1d636b02300a7aa7U),
+	  gu64t2 = G_GINT64_CONSTANT(0xa77a0a30026b631dU);
+#endif
 
   g_print ("TestGLib v%u.%u.%u (i:%u b:%u)\n",
 	   glib_major_version,
@@ -327,10 +346,10 @@ main (int   argc,
   TEST (NULL, sizeof (gint16) == 2);
   g_print ("\nchecking size of gint32: %d", (int)sizeof (gint32));
   TEST (NULL, sizeof (gint32) == 4);
-#ifdef	HAVE_GINT64
+#ifdef	G_HAVE_GINT64
   g_print ("\nchecking size of gint64: %d", (int)sizeof (gint64));
   TEST (NULL, sizeof (gint64) == 8);
-#endif	/* HAVE_GINT64 */
+#endif	/* G_HAVE_GINT64 */
   g_print ("\n");
 
   g_print ("checking g_dirname()...");
@@ -406,6 +425,26 @@ main (int   argc,
          g_error ("Sorted insert failed");
     }
 
+  g_list_free (list);
+  list = NULL;
+
+  for (i = 0; i < 10; i++)
+    list = g_list_prepend (list, &morenums[i]);
+
+  list = g_list_sort (list, my_list_compare_two);
+
+  /*
+  g_print("\n");
+  g_list_foreach (list, my_list_print, NULL);
+  */
+
+  for (i = 0; i < 10; i++)
+    {
+      t = g_list_nth (list, i);
+      if (*((gint*) t->data) != (9 - i))
+         g_error ("Merge sort failed");
+    }
+
   g_list_free (list);
 
   g_print ("ok\n");
@@ -454,6 +493,26 @@ main (int   argc,
   g_slist_foreach (slist, my_list_print, NULL);
   */
 
+  for (i = 0; i < 10; i++)
+    {
+      st = g_slist_nth (slist, i);
+      if (*((gint*) st->data) != (9 - i))
+         g_error("Sorted insert failed");
+    }
+
+  g_slist_free(slist);
+  slist = NULL;
+
+  for (i = 0; i < 10; i++)
+    slist = g_slist_prepend (slist, &morenums[i]);
+
+  slist = g_slist_sort (slist, my_list_compare_two);
+
+  /*
+  g_print("\n");
+  g_slist_foreach (slist, my_list_print, NULL);
+  */
+
   for (i = 0; i < 10; i++)
     {
       st = g_slist_nth (slist, i);
@@ -625,12 +684,32 @@ main (int   argc,
   for (i = 0; i < 10000; i++)
     g_string_append_c (string1, 'a'+(i%26));
 
+#if !(defined (_MSC_VER) || defined (__LCC__))
+  /* MSVC and LCC use the same run-time C library, which doesn't like
+     the %10000.10000f format... */
   g_string_sprintf (string2, "%s|%0100d|%s|%s|%0*d|%*.*f|%10000.10000f",
 		    "this pete guy sure is a wuss, like he's the number ",
 		    1,
 		    " wuss.  everyone agrees.\n",
 		    string1->str,
 		    10, 666, 15, 15, 666.666666666, 666.666666666);
+#else
+  g_string_sprintf (string2, "%s|%0100d|%s|%s|%0*d|%*.*f|%100.100f",
+		    "this pete guy sure is a wuss, like he's the number ",
+		    1,
+		    " wuss.  everyone agrees.\n",
+		    string1->str,
+		    10, 666, 15, 15, 666.666666666, 666.666666666);
+#endif
+
+  g_print ("string2 length = %d...\n", string2->len);
+  string2->str[70] = '\0';
+  g_print ("first 70 chars:\n%s\n", string2->str);
+  string2->str[141] = '\0';
+  g_print ("next 70 chars:\n%s\n", string2->str+71);
+  string2->str[212] = '\0';
+  g_print ("and next 70:\n%s\n", string2->str+142);
+  g_print ("last 70 chars:\n%s\n", string2->str+string2->len - 70);
 
   g_print ("ok\n");
 
@@ -648,7 +727,23 @@ main (int   argc,
 
   g_print ("ok\n");
 
-  g_print ("checking g_strcasecmp...\n");
+  g_print ("checking g_strcasecmp...");
+  g_assert (g_strcasecmp ("FroboZZ", "frobozz") == 0);
+  g_assert (g_strcasecmp ("frobozz", "frobozz") == 0);
+  g_assert (g_strcasecmp ("frobozz", "FROBOZZ") == 0);
+  g_assert (g_strcasecmp ("FROBOZZ", "froboz") != 0);
+  g_assert (g_strcasecmp ("", "") == 0);
+  g_assert (g_strcasecmp ("!#%&/()", "!#%&/()") == 0);
+  g_assert (g_strcasecmp ("a", "b") < 0);
+  g_assert (g_strcasecmp ("a", "B") < 0);
+  g_assert (g_strcasecmp ("A", "b") < 0);
+  g_assert (g_strcasecmp ("A", "B") < 0);
+  g_assert (g_strcasecmp ("b", "a") > 0);
+  g_assert (g_strcasecmp ("b", "A") > 0);
+  g_assert (g_strcasecmp ("B", "a") > 0);
+  g_assert (g_strcasecmp ("B", "A") > 0);
+
+  g_print ("ok\n");
 
   /* g_debug (argv[0]); */
 
@@ -757,10 +852,23 @@ main (int   argc,
   g_print ("ok\n");
 
   g_printerr ("g_log tests:");
-  g_warning ("harmless warning");
+  g_warning ("harmless warning with parameters: %d %s %#x", 42, "Boo", 12345);
   g_message ("the next warning is a test:");
   string = NULL;
   g_print (string);
 
+  g_print ("endian macro tests...");
+#if G_BYTE_ORDER == G_BIG_ENDIAN
+  g_print ("big endian...");
+#else
+  g_print ("little endian...");
+#endif
+  g_assert (GUINT16_SWAP_LE_BE (gu16t1) == gu16t2);  
+  g_assert (GUINT32_SWAP_LE_BE (gu32t1) == gu32t2);  
+#ifdef G_HAVE_GINT64
+  g_assert (GUINT64_SWAP_LE_BE (gu64t1) == gu64t2);  
+#endif
+  g_print ("ok\n");
+
   return 0;
 }
